@@ -2,84 +2,68 @@ use crate::{classify::*, const_interval, interval::*};
 use gmp_mpfr_sys::mpfr;
 use rug::Float;
 
-fn mpfr_fn(
-    f: unsafe extern "C" fn(*mut mpfr::mpfr_t, *const mpfr::mpfr_t, mpfr::rnd_t) -> i32,
-    x: f64,
-    rnd: mpfr::rnd_t,
-) -> f64 {
-    let mut x = Float::with_val(f64::MANTISSA_DIGITS, x);
-    unsafe {
-        f(x.as_raw_mut(), x.as_raw(), rnd);
-        mpfr::get_d(x.as_raw(), rnd)
-    }
-}
-
-fn mpfr_fn2(
-    f: unsafe extern "C" fn(
-        *mut mpfr::mpfr_t,
-        *const mpfr::mpfr_t,
-        *const mpfr::mpfr_t,
-        mpfr::rnd_t,
-    ) -> i32,
-    x: f64,
-    y: f64,
-    rnd: mpfr::rnd_t,
-) -> f64 {
-    let mut x = Float::with_val(f64::MANTISSA_DIGITS, x);
-    let y = Float::with_val(f64::MANTISSA_DIGITS, y);
-    unsafe {
-        f(x.as_raw_mut(), x.as_raw(), y.as_raw(), rnd);
-        mpfr::get_d(x.as_raw(), rnd)
-    }
-}
-
-fn mpfr_fn_si(
-    f: unsafe extern "C" fn(*mut mpfr::mpfr_t, *const mpfr::mpfr_t, i64, mpfr::rnd_t) -> i32,
-    x: f64,
-    y: i64,
-    rnd: mpfr::rnd_t,
-) -> f64 {
-    let mut x = Float::with_val(f64::MANTISSA_DIGITS, x);
-    unsafe {
-        f(x.as_raw_mut(), x.as_raw(), y, rnd);
-        mpfr::get_d(x.as_raw(), rnd)
-    }
-}
-
 macro_rules! mpfr_fn {
     ($mpfr_f:ident, $f_rd:ident, $f_ru:ident) => {
         fn $f_rd(x: f64) -> f64 {
-            mpfr_fn(mpfr::$mpfr_f, x, mpfr::rnd_t::RNDD)
+            mpfr_fn!($mpfr_f(x, RNDD))
         }
 
         fn $f_ru(x: f64) -> f64 {
-            mpfr_fn(mpfr::$mpfr_f, x, mpfr::rnd_t::RNDU)
+            mpfr_fn!($mpfr_f(x, RNDU))
         }
     };
+
+    ($mpfr_f:ident($x:ident, $rnd:ident)) => {{
+        let mut x = Float::with_val(f64::MANTISSA_DIGITS, $x);
+        let rnd = mpfr::rnd_t::$rnd;
+        unsafe {
+            mpfr::$mpfr_f(x.as_raw_mut(), x.as_raw(), rnd);
+            mpfr::get_d(x.as_raw(), rnd)
+        }
+    }};
 }
 
 macro_rules! mpfr_fn2 {
     ($mpfr_f:ident, $f_rd:ident, $f_ru:ident) => {
         fn $f_rd(x: f64, y: f64) -> f64 {
-            mpfr_fn2(mpfr::$mpfr_f, x, y, mpfr::rnd_t::RNDD)
+            mpfr_fn2!($mpfr_f(x, y, RNDD))
         }
 
         fn $f_ru(x: f64, y: f64) -> f64 {
-            mpfr_fn2(mpfr::$mpfr_f, x, y, mpfr::rnd_t::RNDU)
+            mpfr_fn2!($mpfr_f(x, y, RNDU))
         }
     };
+
+    ($mpfr_f:ident($x:ident, $y:ident, $rnd:ident)) => {{
+        let mut x = Float::with_val(f64::MANTISSA_DIGITS, $x);
+        let y = Float::with_val(f64::MANTISSA_DIGITS, $y);
+        let rnd = mpfr::rnd_t::$rnd;
+        unsafe {
+            mpfr::$mpfr_f(x.as_raw_mut(), x.as_raw(), y.as_raw(), rnd);
+            mpfr::get_d(x.as_raw(), rnd)
+        }
+    }};
 }
 
 macro_rules! mpfr_fn_si {
     ($mpfr_f:ident, $f_rd:ident, $f_ru:ident) => {
         fn $f_rd(x: f64, y: i32) -> f64 {
-            mpfr_fn_si(mpfr::$mpfr_f, x, y as i64, mpfr::rnd_t::RNDD)
+            mpfr_fn_si!($mpfr_f(x, y, RNDD))
         }
 
         fn $f_ru(x: f64, y: i32) -> f64 {
-            mpfr_fn_si(mpfr::$mpfr_f, x, y as i64, mpfr::rnd_t::RNDU)
+            mpfr_fn_si!($mpfr_f(x, y, RNDU))
         }
     };
+
+    ($mpfr_f:ident($x:ident, $y:ident, $rnd:ident)) => {{
+        let mut x = Float::with_val(f64::MANTISSA_DIGITS, $x);
+        let rnd = mpfr::rnd_t::$rnd;
+        unsafe {
+            mpfr::$mpfr_f(x.as_raw_mut(), x.as_raw(), $y as i64, rnd);
+            mpfr::get_d(x.as_raw(), rnd)
+        }
+    }};
 }
 
 mpfr_fn!(acos, acos_rd, acos_ru);
@@ -246,7 +230,7 @@ impl Interval {
     );
 
     /// Returns the angle of the point $(\rhs, \self)$ measured counterclockwise from the positive
-    /// $x$-axis in the Euclidean $xy$-plane.
+    /// $x$-axis in the Euclidean plane.
     ///
     /// Tightness: tightest
     pub fn atan2(self, rhs: Self) -> Self {
@@ -458,7 +442,7 @@ impl Interval {
     }
 
     impl_mono_inc!(
-        /// Returns the exponential of `self`.
+        /// Returns `self` raised to the power of $\e$.
         ///
         /// Tightness: tightest
         exp,
@@ -582,7 +566,7 @@ impl Interval {
         }
     }
 
-    /// Returns `self` raised to the power of `rhs`, where `rhs` is an integer.
+    /// Returns `self` raised to the power of `rhs`.
     ///
     /// Tightness: tightest
     pub fn pown(self, rhs: i32) -> Self {
@@ -728,12 +712,14 @@ impl Interval {
 
 macro_rules! impl_dec {
     ($f:ident) => {
+        #[doc = concat!("See [`Interval::", stringify!($f), "`].")]
         pub fn $f(self) -> Self {
             Self::set_dec(self.x.$f(), self.d)
         }
     };
 
     ($f:ident, $f_impl:ident) => {
+        #[doc = concat!("See [`Interval::", stringify!($f), "`].")]
         pub fn $f(self) -> Self {
             let (y, d) = self.x.$f_impl();
             Self::set_dec(y, self.d.min(d))
@@ -743,6 +729,7 @@ macro_rules! impl_dec {
 
 macro_rules! impl_dec2 {
     ($f:ident, $f_impl:ident) => {
+        #[doc = concat!("See [`Interval::", stringify!($f), "`].")]
         pub fn $f(self, rhs: Self) -> Self {
             let (z, d) = self.x.$f_impl(rhs.x);
             Self::set_dec(z, self.d.min(rhs.d.min(d)))
@@ -768,6 +755,7 @@ impl DecInterval {
     impl_dec!(log2, log2_impl);
     impl_dec2!(pow, pow_impl);
 
+    /// See [`Interval::pown`].
     pub fn pown(self, rhs: i32) -> Self {
         let (y, d) = self.x.pown_impl(rhs);
         Self::set_dec(y, self.d.min(d))
